let Validator = require('validator'),
		ObjectID = require('mongodb').ObjectID,
		Util = require('util');

let _transposeUserFields = {
		ID           : '_id',
		Title        : 'title',
		Description  : 'description',
		Flags        : 'flags'
};

function _parseQuery(query, get, _idAsObjectID = true) {
	let q = {};
	for(let k in query) {
		if(k == '$or' && query[k] instanceof Array) {
			let orQueries = [];
			for(let i in query[k]) {
				orQueries[orQueries.length] = _parseQuery(query[k][i]);
			}
			q.$or = orQueries;
		} else {
			let k2 = '';
			if(k in _transposeUserFields) {
				k2 = _transposeUserFields[k];
			} else {
				k2 = k;
			}
			if(query[k] instanceof Array && get) {
			  let qinfo = query[k].queryInformation || {},
			      qlogic = qinfo.logic || "";
			  delete query[k].queryInformation;
			  if(qlogic == "nin") {
			    // Not in
			    q[k2] = {$nin: query[k]};
			  } else if(qlogic == 'or') {
					// Match any
					q[k2] = {$in: query[k]};
				} else {
					// Match all
					q[k2] = {$all: query[k]};
				}
			} else if(k2 == '_id' && query[k] && _idAsObjectID) {
				q[k2] = new ObjectID(query[k]);
			} else {
				q[k2] = query[k];
			}
		}
	}
	return q;
};

module.exports = class RoleStore {
	
	constructor() {};

  /**
   * Corresponds to Role.delete
   */
	delete(_id, fn) {
		Site.Configuration.DB.Mongo.collection('d_role_definition').remove({_id: _id}, (err, res) => {
			return fn && fn(err, res.results);
		});
	};

	get(args, fn) {
		let q = _parseQuery(args.query, true, true),
		    r = _parseQuery(args.return || {}, true, false),
		    s = _parseQuery(args.sort || {}, true, false),
		    l = args.limit && Validator.isInt(args.limit.toString()) ? args.limit : 0;
		Site.Configuration.DB.Mongo.collection('d_role_definition').find(q, r).limit(l).sort(s).toArray((err, results) => {
			return fn && fn(err, results);
		});
	};

  /**
   * Corresponds to Role.save
   */
	save(_id, query, fn) {
		let q = _parseQuery(query, false);
		if(!_id) {
			delete q._id;
			Site.Configuration.DB.Mongo.collection('d_role_definition').insert(q, (err, res) => {
				if(err) {
				  res = false;
				} else {
				  res = q;
				}
				return fn && fn(err, res);
			});
		} else {
			Site.Configuration.DB.Mongo.collection('d_role_definition').update({_id: _id}, q, {upsert: false}, (err, res) => {
				if(err) {
				  res = false;
				} else {
				  res = q;
				}
				return fn && fn(err, res);
			});
		}
	};

}
