let EventEmitter = require('events'),
		util = require('util'),
		roleStore = null,
		properties = new WeakMap();

const FLAG_ACTIVE = 'active';

let sanatizeProperties = {
  _id: true
};

function propertiesSanatizer(s, p) {
  for(let k in s) {
    if(!k in p) continue;
    if(!(typeof s[k] == 'boolean')) {
      propertiesSanatizer(s[k], p[k]);
    } else if(s[k]) {
      delete p[k];
    }
  } 
}

module.exports = (options) => {

  options = options || {};
  
	if(!roleStore) {
		if(!options.store) {
			console.log('xceling-role error: no \'store\' option specified, cannot continue.');
			return null;
		}
		roleStore = new (require(`./stores/${options.store}-store`))();
	}
	
	class Role extends EventEmitter {
		constructor(options) {
			super();
			let _properties = {
				_id: false,
				title: null,
				description: null,
				flags: []
			}
			properties.set(this, Object.assign(_properties, options || {}));
		}

		/* Instance variables, getters and setters */
		get Properties() {
		  try {
		    return Object.assign({}, properties.get(this));
		  } catch(e) {
		    return {};
		  }
		};
		get SanatizedProperties() {
		  try {
		    let obj = Object.assign({}, properties.get(this));
		    propertiesSanatizer(sanatizeProperties, obj);
		    return obj;
      } catch(e) {
        return {}; 
      }		    
		};
		get ID() {
			try {
				return properties.get(this)._id.toHexString();
			} catch(e) {
				return false;
			}
		};
		get Title() {
			try {
				return properties.get(this).title;
			} catch(e) {
				return null;
			}
		};
		set Title(v) {
			try {
				properties.get(this).title = v;
			} catch(e) {
			}
		};
		get Description() {
			try {
				return properties.get(this).description;
			} catch(e) {
				return null;
			}
		};
		set Description(v) {
			try {
				properties.get(this).description = v;
			} catch(e) {
			}
		};
		get Flags() {
		  try {
		    return properties.get(this).flags;
		  } catch(e) {
		    return [];
		  } 
		};
		set Flags(v) {
		  try {
		    properties.get(this).flags = v instanceof Array ? v : [];
		  } catch(e) {
		  } 
		};
		/* End instance variables, getters and setters */
		/* Instance methods */

		delete(fn) {
			roleStore.delete(properties.get(this)._id, (err, res) => {
				if(!err) {
					properties.set(this, {});
				}
				return fn && fn(err, this);
			});
		}
		
		save(fn) {
		  let _props = properties.get(this);
			roleStore.save(_props._id, _props, (err, response) => {
			  let errMsg = null,
			      errName = null,
			      errNew = null;
			  if(!err) {
			    properties.set(this, response);
			  } else {
			    switch(err.code) {
			      default:
			        errName = `General Error`;
			        errMsg = `Your entry has not been saved. An error occurred while attempting to save this entry: ${err.errmsg}`;
			    }
			    errNew = new Error(errMsg);
			    errNew.message = errMsg;
			    errNew.name = errName;
			    errNew.type = "xceling-menu/xceling-menu.js";
			  }
				return fn && fn(errNew, this);
			});
		}
		/* End instance methods */
		
		// Role constants
		static get FLAG_ACTIVE() {
			return FLAG_ACTIVE;
		};
		
		static get Flags() {
		  return [{name: 'Active', value: FLAG_ACTIVE, description: 'Marks this entry as being active.'}]; 
		}

    /**
     * Required for module hot swapping
     */
  	static get ModulePath() {
  	  return __filename;
  	}
  	
  	// End Role constants
		
		/**
		 *
		 * @param {Object} args, required
		 * @param {Function} fn, required
		 * @api public
		 */
		static getAllRoleDataArray(args, fn) {
		  roleStore.get(args, (err, results) => {
		    return fn && fn(err, results);
		  });
		}
		
		/**
		 * Attempts to retrieve a single Role instance according to supplied search items in args. This method
		 * returns by a defined callback function (fn) only. fn must be defined as: function(err, template)
		 * where {String} err should be null for no error else a descriptive string, and {User} template
		 * should be a User object upon return.
		 *
		 * @param {Object} args, required
		 * @param {Function} fn, required
		 * @api public
		 */
		static getSingleObject(args, fn) {
			roleStore.get({query:args}, (err, results) => {
				if(!err && results.length == 1) {
					return fn && fn(null, new Role(results[0]));
				} else {
				  return fn && fn(err, new Role());
				}
			});
		}
  };

	return Role;
}
